@@ -14,11 +14,13 @@
  Modified for Kommander:
   (C) 2002-2003 Marc Britton <consume@optusnet.com.au>
   (C) 2004      Michal Rudolf <mrudolf@kdewebdev.org>
+  (C) 2008      Andras Mantia <amantia@kde.org>
 
 **********************************************************************/
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <sys/stat.h>
 
 #include "actiondnd.h"
 #include "actioneditorimpl.h"
@@ -38,7 +40,7 @@
 #include "pixmapchooser.h"
 #include "propertyeditor.h"
 #include "resource.h"
-#include "specials.h"
+#include "specialinformation.h"
 #include "widgetdatabase.h"
 #include "widgetfactory.h"
 #include "wizardeditorimpl.h"
@@ -84,6 +86,7 @@
 #include <ktoolbar.h>
 #include <kurl.h>
 #include <kparts/partmanager.h>
+#include <kio/netaccess.h>
 
 extern QMap<QWidget*, QString> *qwf_functions;
 extern QMap<QWidget*, QString> *qwf_forms;
@@ -192,6 +195,9 @@ MainWindow::MainWindow(bool asClient)
   statusBar()->setSizeGripEnabled(true);
   SpecialInformation::registerSpecials();
 
+  backupTimer = new QTimer(this);
+  connect(backupTimer, SIGNAL(timeout()), this, SLOT(slotCreateBackups()));
+  backupTimer->start(1000*60*5); //fire it every five minutes
   //createGUI(0);
 
 }
@@ -365,6 +371,33 @@ int MainWindow::currentTool() const
   return QString::fromLatin1(actionCurrentTool->name()).toInt();
 }
 
+void MainWindow::slotCreateBackups()
+{
+//create a backup of the opened forms
+  QWidgetList windows = qworkspace->windowList(QWorkspace::StackingOrder);
+  for (int i = 0; i < (int)windows.count(); ++i)
+  {
+    FormWindow* form = dynamic_cast<FormWindow*>(windows.at(i));
+    if (!form || !form->formFile())
+      continue;
+    QString fileName = form->formFile()->fileName();
+    QString backupName = fileName + ".backup";
+    bool modified = form->formFile()->isModified();
+    if (form->formFile()->hasTempFileName())
+      continue; //no need to create a backup
+    if (!modified)
+      continue;
+    
+    form->formFile()->setFileName(backupName);  
+    form->formFile()->setModified(true);
+    if (!form->formFile()->save(false))
+    {
+      KMessageBox::error(this, i18n("<qt>Cannot create backup file <i>%1</i>.</qt>").arg(backupName));
+    }
+    form->formFile()->setFileName(fileName);  
+    form->formFile()->setModified(modified); 
+  }
+}
 
 void MainWindow::runForm()
 {
@@ -376,8 +409,46 @@ void MainWindow::runForm()
   FormWindow* form = activeForm();
   if (!form || !form->formFile())
     return;
+
+  QObjectList *editors = queryList("AssocTextEditor");
+  QObjectListIt it(*editors);
+  QObject *editor;
+
+  while ((editor = it.current()) != 0L) 
+  {
+    ++it;
+    static_cast<AssocTextEditor*>(editor)->save();
+  }
+  delete editors;  
+
+  if (form->formFile()->hasTempFileName())
+  {
+    if (!form->formFile()->saveAs())
+      return;
+  }
+
+  m_fileName = form->formFile()->fileName();
+  m_backupName = m_fileName + ".running";
+  m_modified = form->formFile()->isModified();
+    
+  bool readOnlyFile = !QFileInfo(m_fileName).isWritable();
+  struct stat statbuf;
+  ::stat(m_fileName.local8Bit(), &statbuf);
+  if (!readOnlyFile && !KIO::NetAccess::file_copy(KURL::fromPathOrURL(m_fileName), KURL::fromPathOrURL(m_backupName), statbuf.st_mode, true))
+  {
+    KMessageBox::error(this, i18n("<qt>Cannot create temporary file <i>%1</i>.</qt>").arg(m_backupName));
+    return;
+  }
+  form->formFile()->setFileName(m_fileName);  
+  if (!readOnlyFile || m_modified)
+    form->formFile()->setModified(true);
   if (form->formFile()->save(false))
   {
+    if (!readOnlyFile && !KIO::NetAccess::file_copy(KURL::fromPathOrURL(m_fileName), KURL::fromPathOrURL(m_fileName + ".backup"), statbuf.st_mode, true))
+    {
+      KMessageBox::error(this, i18n("<qt>Cannot create backup file <i>%1</i>.</qt>").arg(m_fileName + ".backup"));
+    }
+    ::chmod(m_fileName.local8Bit(), S_IRWXU);
     KProcess* process = new KProcess;
     process->setUseShell(true);
     (*process) << "kmdr-executor" << QString("\"%1\"").arg(form->formFile()->fileName());
@@ -396,6 +467,15 @@ void MainWindow::closeRunningForm(KProcess* process)
 {
   previewing = false;
   delete process;
+ 
+  if (QFileInfo(m_backupName).exists())
+  {
+    struct stat statbuf;
+    ::stat(m_fileName.local8Bit(), &statbuf);
+    FormWindow* form = activeForm();
+    KIO::NetAccess::file_move(KURL::fromPathOrURL(m_backupName), KURL::fromPathOrURL(m_fileName), statbuf.st_mode, true);
+    form->formFile()->setModified(m_modified);
+  }
 }
 
 void MainWindow::showProperties(QObject *o)
@@ -476,9 +556,8 @@ bool MainWindow::eventFilter(QObject *o, QEvent *e)
       if (((QMouseEvent*)e)->button() == MidButton && dynamic_cast<KommanderWidget *>(o))
       {
         AssocTextEditor *editor = new AssocTextEditor((QWidget*)o, formWindow(), propertyEditor, m_partManager,
-             this, "AssocTextEditor", false);
-         editor->exec();
-         delete editor;
+             this, "AssocTextEditor", false); //deletes itself!
+         editor->show();
          ((QKeyEvent*)e)->accept();
           return true;
       }
@@ -935,7 +1014,17 @@ void MainWindow::setupRMBSpecialCommands(QValueList<int> &ids, QMap<QString, int
         ids << (id = rmbWidgets->insertItem(i18n("Add Page"), -1, 0));
         commands.insert("add", id);
     }
-    if (WidgetFactory::hasSpecialEditor(WidgetDatabase::idFromClassName(WidgetFactory::classNameOf(w))))
+     if (w->inherits("QToolBox")) {
+        if (ids.isEmpty())
+            ids << rmbWidgets->insertSeparator(0);
+        if (((QToolBox*)w)->count() > 1) {
+            ids << (id = rmbWidgets->insertItem(i18n("Delete Page"), -1, 0));
+            commands.insert("remove", id);
+        }
+        ids << (id = rmbWidgets->insertItem(i18n("Add Page"), -1, 0));
+        commands.insert("add", id);
+    }
+   if (WidgetFactory::hasSpecialEditor(WidgetDatabase::idFromClassName(WidgetFactory::classNameOf(w))))
     {
         if (ids.isEmpty())
             ids << rmbWidgets->insertSeparator(0);
@@ -1054,9 +1143,8 @@ void MainWindow::handleRMBSpecialCommands(int id, QMap<QString, int> &commands,
     if(id == commands["assoc"])
     {
          AssocTextEditor *editor = new AssocTextEditor(w, formWindow(), propertyEditor, m_partManager,
-             this, "AssocTextEditor", false);
-         editor->exec();
-         delete editor;
+             this, "AssocTextEditor", false); //deletes itself!
+         editor->show();
     }
 
     if (w->inherits("QTabWidget")) {
@@ -1077,6 +1165,24 @@ void MainWindow::handleRMBSpecialCommands(int id, QMap<QString, int> &commands,
             }
         }
     }
+    if (w->inherits("QToolBox")) {
+        QToolBox *tw = (QToolBox*)w;
+        if (id == commands[ "add" ]) {
+            AddToolBoxPageCommand *cmd = new AddToolBoxPageCommand(i18n("Add Page to %1").arg(tw->name()), formWindow(),
+                                                            tw, "Page");
+            formWindow()->commandHistory()->addCommand(cmd);
+            cmd->execute();
+        } else if (id == commands[ "remove" ]) {
+            if (tw->currentItem()) {
+                EditorToolBox *dtw = (EditorToolBox*)tw;
+                DeleteToolBoxPageCommand *cmd = new DeleteToolBoxPageCommand(i18n("Delete Page %1 of %2").
+                                                                      arg(dtw->pageTitle()).arg(tw->name()),
+                                                                      formWindow(), tw, tw->currentItem());
+                formWindow()->commandHistory()->addCommand(cmd);
+                cmd->execute();
+            }
+        }
+    }
     if (WidgetFactory::hasSpecialEditor(WidgetDatabase::idFromClassName(WidgetFactory::classNameOf(w)))) {
         if (id == commands[ "edit" ])
             WidgetFactory::editWidget(WidgetDatabase::idFromClassName(WidgetFactory::classNameOf(w)), this, w, formWindow());
@@ -1089,9 +1195,8 @@ void MainWindow::handleRMBSpecialCommands(int id, QMap<QString, int> &commands,
     if(id == commands["assoc"])
     {
          AssocTextEditor *editor = new AssocTextEditor(fw->mainContainer(), formWindow(), propertyEditor, m_partManager,
-             this, "AssocTextEditor", false);
-         editor->exec();
-         delete editor;
+             this, "AssocTextEditor", false); //deletes itself!
+         editor->show();
     }
 
 
@@ -1718,7 +1823,7 @@ QString MainWindow::whatsThisFrom(const QString&)
 
 void MainWindow::slotActivePartChanged(KParts::Part * part)
 {
-  kdDebug(24000) << "ActivePartChanged" << part << endl; 
+//   kdDebug(24000) << "ActivePartChanged" << part << endl; 
   if (part) // if part == 0L the pointer m_oldKTextEditor is not useable
   {
   //  guiFactory()->removeClient(part);
