@@ -62,7 +62,7 @@ virtual QStringList children(const QString& parent, bool recursive) = 0;
 virtual QString global(const QString& variableName) = 0;
 virtual void setGlobal(const QString& variableName, const QString& value) = 0;
 virtual void setMaximum(const QString &widgetName, int value) = 0;
-virtual void execute(const QString &widgetName) = 0;   
+virtual QString execute(const QString &widgetName) = 0;   
 virtual void cancel(const QString &widgetName) = 0;   
 virtual int count(const QString &widgetName) = 0;
 virtual int currentColumn(const QString &widgetName) = 0;
