@@ -16,6 +16,8 @@
 
 #include "expression.h"
 
+#include <klocale.h>
+
 Expression::Expression() : m_start(0), m_error(false)
 {
 }
@@ -89,7 +91,7 @@ QString Expression::next() const
   if (m_start < m_parts.count())
     return m_parts[m_start].toString();
   else 
-    return QString::null;
+    return QString();
 }
 
 bool Expression::validate()
@@ -98,7 +100,6 @@ bool Expression::validate()
     setError();
   return !m_error;
 }
-
 Expression::Type Expression::commonType(const QVariant v1, const QVariant v2) const
 {
   if (v1.type() == QVariant::String || v2.type() == QVariant::String)
@@ -186,20 +187,30 @@ QVariant Expression::parseMultiply()
     QVariant value2 = parseBracket();
     Type mode = commonType(value, value2);
     if (op == "*")
+    {
       if (mode == TypeDouble)
         value = value.toDouble() * value2.toDouble();
       else
         value = value.toInt() * value2.toInt();
+    }
     else if (op == "/")
-      if (mode == TypeDouble)
+    {
+      if (value2.toInt() == 0)
+        return i18n("error");
+      if (mode == TypeDouble || value.toInt() != value.toInt() / value2.toInt() * value2.toInt())
         value = value.toDouble() / value2.toDouble();
       else
         value = value.toInt() / value2.toInt();
+    }
     else 
+    {
+      if (value2.toInt() == 0)
+        return i18n("error");
       if (mode == TypeDouble)
         value = value.toDouble() / value2.toInt();
       else
         value = value.toInt() / value2.toInt();
+    }
     op = next();
   }
   return value;
@@ -210,7 +221,7 @@ QVariant Expression::parseAdd()
   if (!validate()) return -1;
   QVariant value = parseMultiply();
   QString op = next();
-  while (next() == "+" || next() == "-")
+  while (op == "+" || op == "-")
   {
     m_start++;
     QVariant value2 = parseMultiply();
@@ -225,6 +236,7 @@ QVariant Expression::parseAdd()
         value = value.toDouble() - value2.toDouble();
       else
         value = value.toInt() - value2.toInt();
+    op = next();
   }
   return value;
 }
