@@ -23,8 +23,8 @@
 
 /* OTHER INCLUDES */
 #include <kommanderwidget.h>
-#include "kommander_export.h"
-#include "myprocess.h"
+#include <kommander_export.h>
+#include <myprocess.h>
 
 class QShowEvent;
 class KOMMANDER_EXPORT Wizard : public QWizard, public KommanderWidget
@@ -39,7 +39,7 @@ class KOMMANDER_EXPORT Wizard : public QWizard, public KommanderWidget
   Q_PROPERTY(QString helpActionText READ helpActionText WRITE setHelpActionText)
 
 public:
-  Wizard(QWidget *, const char *, bool=TRUE, int=0);
+  Wizard(QWidget *, const char *, bool = true, int = 0);
   ~Wizard();
 
   virtual bool isKommanderWidget() const;
@@ -70,11 +70,13 @@ signals:
   void widgetOpened();
   void widgetTextChanged(const QString &);
   void finished();
+  void contextMenuRequested(int xpos, int ypos);
 protected:
+  void showEvent( QShowEvent *e );
+  void contextMenuEvent( QContextMenuEvent * e );
+
   HelpAction m_helpAction;
   QString m_helpActionText;
-  void showEvent( QShowEvent *e );
-private:
 };
 
 #endif
