@@ -187,7 +187,7 @@ validateSource(xmlChar ** url, long *lineNo)
                     walkChildNodes((xmlHashScanner) scanForNode, searchInf,
                                    searchData->node);
                     if (!searchInf->found) {
-                        xsldbgGenericErrorFunc(i18n("Warning: Breakpoint for file %1 at line %2 does not seem to be valid.\n").arg(xsldbgUrl(*url)).arg(*lineNo));
+                        xsldbgGenericErrorFunc(i18n("Warning: Breakpoint for file \"%1\" at line %2 does not seem to be valid.\n").arg(xsldbgUrl(*url)).arg(*lineNo));
                     }
 
                     *lineNo = searchData->lineNo;
@@ -211,7 +211,7 @@ validateSource(xmlChar ** url, long *lineNo)
         } else{
             xsldbgGenericErrorFunc(i18n("Error: Unable to find a stylesheet file whose name contains %1.\n").arg(xsldbgUrl(*url)));
 	    if (lineNo){
-	      xsldbgGenericErrorFunc(i18n("Warning: Breakpoint for file %1 at line %2 does not seem to be valid.\n").arg(xsldbgUrl(*url)).arg(*lineNo));
+	      xsldbgGenericErrorFunc(i18n("Warning: Breakpoint for file \"%1\" at line %2 does not seem to be valid.\n").arg(xsldbgUrl(*url)).arg(*lineNo));
 	    }
 	}
     }
@@ -298,7 +298,7 @@ validateData(xmlChar ** url, long *lineNo)
 
         if (!searchInf->found) {
 	  if (lineNo){
-             xsldbgGenericErrorFunc(i18n("Warning: Breakpoint for file %1 at line %2 does not seem to be valid.\n").arg(xsldbgUrl(*url)).arg(*lineNo));
+             xsldbgGenericErrorFunc(i18n("Warning: Breakpoint for file \"%1\" at line %2 does not seem to be valid.\n").arg(xsldbgUrl(*url)).arg(*lineNo));
 	  } else{
              xsldbgGenericErrorFunc(i18n("Error: Unable to find a data file whose name contains %1.\n").arg(xsldbgUrl(*url)));
 	  }
@@ -546,7 +546,9 @@ xslDbgShellBreak(xmlChar * arg, xsltStylesheetPtr style,
 		  } else if (templ->match){
 		    if ((xmlStrEqual(templ->match, name) &&
 			 (!modeURI || xmlStrEqual(templ->modeURI, 
-						  modeURI))))
+						  modeURI)) &&
+			 (!mode || xmlStrEqual(templ->mode, 
+						  mode))))
 			found = 1;		     
 		  }else{
 			if(xmlStrEqual(templ->name, name) && 
@@ -589,7 +591,7 @@ xslDbgShellBreak(xmlChar * arg, xsltStylesheetPtr style,
 			    }
 			}else{
 			    if (xsldbgValidateBreakpoints != BREAKPOINTS_BEING_VALIDATED){
-				xsldbgGenericErrorFunc(i18n("Warning: Breakpoint exits for file %1 at line %2.\n").arg(xsldbgUrl(tempUrl)).arg(templateLineNo));
+				xsldbgGenericErrorFunc(i18n("Warning: Breakpoint exits for file \"%1\" at line %2.\n").arg(xsldbgUrl(tempUrl)).arg(templateLineNo));
 			    }
 			    validatedBreakPoints++;
 			}
@@ -637,7 +639,7 @@ xslDbgShellBreak(xmlChar * arg, xsltStylesheetPtr style,
 
     if (!result) {
         if (url)
-            xsldbgGenericErrorFunc(i18n("Error: Failed to add breakpoint for file %1 at line %2.\n").arg(xsldbgUrl(url)).arg(lineNo));
+            xsldbgGenericErrorFunc(i18n("Error: Failed to add breakpoint for file \"%1\" at line %2.\n").arg(xsldbgUrl(url)).arg(lineNo));
         else
             xsldbgGenericErrorFunc(i18n("Error: Failed to add breakpoint.\n"));
     }
@@ -691,7 +693,7 @@ xslDbgShellDelete(xmlChar * arg)
                         } else if (validateData(&url, &lineNo))
                             breakPtr = breakPointGet(url, lineNo);
                         if (!breakPtr || !breakPointDelete(breakPtr)){
-                            xsldbgGenericErrorFunc(i18n("Error: Breakpoint does not exist for file %1 at line %2.\n").arg(xsldbgUrl(url)).arg(lineNo));
+                            xsldbgGenericErrorFunc(i18n("Error: Breakpoint does not exist for file \"%1\" at line %2.\n").arg(xsldbgUrl(url)).arg(lineNo));
                         }else{
                             result = 1;
 			}
@@ -722,10 +724,10 @@ xslDbgShellDelete(xmlChar * arg)
         if (breakPtr) {
             result = breakPointDelete(breakPtr);
             if (!result) {
-                xsldbgGenericErrorFunc(i18n("Error: Unable to delete breakpoint at template %1.\n").arg((char*)arg));
+                xsldbgGenericErrorFunc(i18n("Error: Unable to delete breakpoint at template %1.\n").arg(xsldbgText(arg)));
             }
         } else{
-            xsldbgGenericErrorFunc(i18n("Error: Breakpoint at template \"%1\" does not exist.\n").arg((char*)arg));
+            xsldbgGenericErrorFunc(i18n("Error: Breakpoint at template \"%1\" does not exist.\n").arg(xsldbgText(arg)));
 	}
     }
     if (!result) 
@@ -810,7 +812,7 @@ xslDbgShellEnable(xmlChar * arg, int enableType)
                         if (breakPtr){
                             result = breakPointEnable(breakPtr, enableType);
                         }else{
-                            xsldbgGenericErrorFunc(i18n("Error: Breakpoint does not exist for file %1 at line %2.\n").arg(xsldbgUrl(url)).arg(lineNo));
+                            xsldbgGenericErrorFunc(i18n("Error: Breakpoint does not exist for file \"%1\" at line %2.\n").arg(xsldbgUrl(url)).arg(lineNo));
 			}
                         xmlFree(url);
                     }
@@ -839,7 +841,7 @@ xslDbgShellEnable(xmlChar * arg, int enableType)
         if (breakPtr) {
             result = breakPointEnable(breakPtr, enableType);
         } else
-            xsldbgGenericErrorFunc(i18n("Error: Breakpoint at template \"%1\" does not exist.\n").arg((char*)arg));
+            xsldbgGenericErrorFunc(i18n("Error: Breakpoint at template \"%1\" does not exist.\n").arg(xsldbgText(arg)));
     }
 
     if (!result)
