@@ -523,7 +523,7 @@ xslDbgCd(xsltTransformContextPtr styleCtxt, xmlShellCtxtPtr ctxt,
 
                     /* quickly find a template */
                     /* skip any white spaces */
-                    while (IS_BLANK(arg[offset]))
+                    while (_IS_BLANK(arg[offset]))
                         offset++;
 
                     templateNode =
@@ -1989,7 +1989,7 @@ shellPrompt(xmlNodePtr source, xmlNodePtr doc, xmlChar * filename,
 		    cmdResult = 0;
 		}
 #else
-	        xsldbgGenericErrorFunc(i18n("Warning: The %1 command is disbled\n").arg("lang"));
+	        xsldbgGenericErrorFunc(i18n("Warning: The %1 command is disabled\n").arg("lang"));
 		cmdResult = 1;
 #endif
 		break;
