@@ -503,7 +503,7 @@ void QExtFileInfo::slotNewEntries(KIO::Job *job, const KIO::UDSEntryList& udsLis
         KURL u = item->url();
         QString linkDest = item->linkDest();
         kdDebug(24000) << "Got link: " << name << " Points to:" << linkDest << endl;
-        if (linkDest.startsWith("."))
+        if (linkDest.startsWith("./") || linkDest.startsWith("../") )
         {
           u.setPath(u.directory(false, true) + linkDest);
           u.cleanPath();
